//! Code Name rama - Rede an mein Auto

/// ch341-uart converter now attached to ttyUSB0
extern crate serial;

use std::env;
use std::io;
use std::time::Duration;
use std::thread::sleep;

use std::io::prelude::*;
use serial::prelude::*;

use ansi_term::Colour::Red as Reads;
//use ansi_term::Colour::Blue as Writes;
use ansi_term::Colour::Cyan as Writes;

fn main() {
    #[cfg(unix)]
    let device = "/dev/ttyUSB0";
    #[cfg(windows)]
    let device = "COM3";
    
    println!("connecting to {:?}", device);
    
    let mut port = serial::open(device).unwrap();
    interact(&mut port).expect(&format!("unable to interact with {}", device));
}

fn interact<T: SerialPort>(port: &mut T) -> io::Result<()> {
    (port.reconfigure(&|settings| {
        (settings.set_baud_rate(serial::Baud38400))?;
        //(settings.set_baud_rate(serial::Baud9600))?;
        settings.set_char_size(serial::Bits8);
        settings.set_parity(serial::ParityNone);
        settings.set_stop_bits(serial::Stop1);
        //settings.set_flow_control(serial::FlowNone);
        settings.set_flow_control(serial::FlowSoftware);
        Ok(())
    }))?;

    port.set_timeout(Duration::from_millis(1000))?;

    send_pkg(port, b"\n")?;

    let mut buf: Vec<u8> = (0..255).collect();

    //print_bin(port.read(&mut buf[..])?, &buf);
    read_pkg(port, &mut buf)?;

    //(port.write(&buf[..]))?;
    send_pkg(port, b"ATZ")?;
    //print_bin(port.read(&mut buf[..])?, &buf);
    read_pkg(port, &mut buf)?;

    send_pkg(port, b"ATE0")?;
    read_pkg(port, &mut buf)?;
    send_pkg(port, b"ATL0")?;
    read_pkg(port, &mut buf)?;
    send_pkg(port, b"ATSP A0")?;
    read_pkg(port, &mut buf)?;

    
    send_pkg(port, b"0101")?;
    read_pkg(port, &mut buf)?;
    read_pkg(port, &mut buf)?;
    read_pkg(port, &mut buf)?;

    Ok(())
}

fn print_bin(len: usize, buf: &[u8]) -> String {
    let slice = &buf[0..len];
    let mut ascii = String::new();
    ascii.extend(slice.iter().filter(|&&b| b.is_ascii() && b != b'\r' && b != b'\n').map(|b| char::from(*b)));
    let hex = slice.iter().map(|c| format!("{:x}", c)).collect::<Vec<_>>();
    format!("{} :::::: {:?}", ascii, hex)
}

fn send_pkg<T: SerialPort>(port: &mut T, slice: &[u8]) -> io::Result<usize> {
    let stop = b"\r";
    //let stop = b"\n";
    let stop = b"\n\r";
    //let stop = b"\r\n";

    let  buf = &mut Vec::with_capacity(slice.len() + stop.len());
    buf.extend_from_slice(slice);
    buf.extend_from_slice(stop);

    println!("{}", Writes
        .bold()
        //.on(ansi_term::Colour::Blue)
        .paint(format!("(>>) {}", print_bin(buf.len(), buf) )));
    flush()?;

    let n = port.write(buf)?;
    port.flush()?;
    sleep(Duration::from_millis(250));
    Ok(n)
}

fn read_pkg<T: SerialPort>(port: &mut T, buf: &mut Vec<u8>) -> io::Result<usize> {
    let a = &mut buf[..];
    assert!(a.len() > 0, "read_pkg input buffer zero sized");
    //println!("read buf len: {}", a.len());
    let len = port.read(a)?;
    println!("{}", Reads
        .bold()
        .paint(format!("(<<) {}", print_bin(len, &buf)))
    );
    Ok(len)
}

fn flush() -> std::io::Result<()> {
    //use std::io::Write;
    std::io::stdout().flush()
}
